# Threatray IDA Plugin

The Threatray IDA Plugin allows to trigger certain functionality of Threatray directly within IDA.
The following features are available:

- **Signature Inspection**: List all threats found by Threatray for each function in a table as well as a comment in the
  function view.
- **Function Retro-hunt**: Retro-hunt for one or multiple functions and show all matching functions and their respective
  binaries and analyses in Threatray.

Make sure that the sample you've opened with IDA has already been analyzed by Threatray. Either submit it for
analysis first, or download the sample (or memory dump) from a Threatray report. For Signature Inspection, the sample
has to exist either in your private repository or in Threatrays public repository.
For Function Retro-hunting, the sample has to exist in your private repository.

The plugin supports IDA 7.6+ on Windows and Linux and requires Python 3.8+ to be installed.

## Installation

1. Install the Python requirements (`python3 -m pip install -r requirements.txt`)
2. Run `generate_api_secrets.py`, which will create `threatray_api_secrets.json` file inside the
   plugin (`plugin/threatray_ida_plugin/libs/api`) folder. If you have created a secrets file previously in
   the `~/.threatray` folder, you can skip this step.
3. Copy the files from the `plugin` folder into the plugin folder (`plugins`) where your IDA is installed.

If you see any errors related to importing dependencies (like `requests`, or `authlib`) while opening IDA, then make
sure that the interpreter of IDA is using the system interpreter (`idapyswitch` can be used to switch the
interpreter). Alternatively, try to install the dependencies globally with
`sudo -H python3 -m pip install -r requirements.txt`, or likewise on Windows.

## How to use it

Once a sample is opened in IDA, the plugins functionalities can be reached via `View/Threatray`. Function Retro-hunt
requires an active Function name view and selection of at least one function (see below).
They can also be triggered with right-click and through the `Threatray` menu item.
Additionally, Function Retro-hunt can also be called with right-click in the Disassembly view through the `Threatray`
menu item.

### Signature Inspection

Signature Inspection will fetch and show the threats for each function that Threatray detected in a table as well as a
comment in the function view.

1. Click `View/Threatray/Signature Inspection`
2. Select the desired settings for the Signature Inspection call
    1. The option "Show only functions with signature hits" filters out functions without threats (e.g. first row in the
       following picture)
    2. The option "Color function with signature hits (& their xrefs)" colors each function and its xrefs. Each threat
       gets a different color and if a function has multiple threats, the most prevalent one is taken.
    3. The option "Manually select the signature hits that are to be colored" allows you to select which signature hits
       are to be colored in which color. The selection is made in an additional window after the results have been
       retrieved.

If functions are colored and there are several different signature hits for a function, the color of the most prevalent
signature is selected.

![Signature inspection results](./images/signature_inspection.png)<br />
Signature Inspection result view. It is possible to jump to the function view by double-clicking on the address (first
column).

![Signature inspection comment](./images/signature_inspection_comment.png)<br />
Signature Inspection result as a comment.

### Function Retro-hunt

Function Retro-hunt allows to hunt for one or multiple functions and lists the matching binaries within IDA.

This functionality can be used when the `Function name` view is activated and at least one function is selected.
It is also possible to access the feature from the context menu.

![](./images/function_name_view_context.png)

The following workflow will be triggered:

1. Enter the threshold value for the function retro-hunt if more than one function was selected. This indicates the
   number of input function that must match with a binary file for it to be returned.
2. It will execute the query and present the result in a table. This might take a while.

![Function Retro-hunt Result View](./images/function_retrohunt.png)<br />
Function Retro-hunt results view.
