import getpass
import json
import sys
from pathlib import Path

from plugin.threatray_ida_plugin.libs.oauth2 import Oauth2, Credential

secrets_file = Path('plugin') / 'threatray_ida_plugin' / 'libs' / 'api' / 'threatray_api_secrets.json'
if secrets_file.exists():
    print(f'Secrets file {secrets_file} already exists, overwrite? (y/n)')
    if 'y' not in input().lower():
        print('Exiting.')
        sys.exit(0)

    secrets_file.unlink(missing_ok=True)

print('Please enter the required information:')
realm = input('Realm (from: <realm>.analysis.threatray.com): ')
user = input('Username: ')
password = getpass.getpass()
if not password:
    print('No password given, exiting.')
    sys.exit(1)

oauth2 = Oauth2(realm)
secret_token = oauth2.get_secret_token(Credential(username=user, password=password))

with open(secrets_file, 'w') as f:
    json.dump({'realm': realm, 'secret_token': secret_token}, f)

print(f'Created secrets file: {secrets_file}')
