from typing import List

import idaapi

from threatray_ida_plugin.application.function_retrohunt_service import FunctionRetrohuntService
from threatray_ida_plugin.handlers.threatray_handler import ThreatrayHandler
from threatray_ida_plugin.libs.constants import PLUGIN_ID, PLUGIN_NAME, OKAY_RESPONSE
from threatray_ida_plugin.libs.logger import get_log
from threatray_ida_plugin.views.message_box import MessageBox

log = get_log()


class FunctionRetrohuntDisassemblyViewHandler(ThreatrayHandler):
    action_name = f'{PLUGIN_ID}:function_retrohunt_disassembly_view'

    def __init__(self, service: FunctionRetrohuntService):
        super().__init__(
            action_name=self.action_name,
            action_text='Function Retrohunt',
            wanted_hotkey='',
            help='Retrohunt function and show all matching functions and their respective code regions and analyses.')
        self.__service = service

    def activate(self, _) -> int:
        current_address: int = idaapi.get_screen_ea()
        current_function = idaapi.get_func(current_address)
        if not current_function:
            msg = f'Invalid address. The selected address ({hex(current_address)}) is not within a function.'
            MessageBox.create('Warning - Invalid address', msg)
            log.warning(msg)
            return 0
        selected_functions: List[int] = [current_function.start_ea]
        self.__service.workflow(selected_functions)
        return OKAY_RESPONSE

    def update(self, ctx) -> int:
        if ctx.widget_type == idaapi.BWN_DISASM:
            return idaapi.AST_ENABLE_FOR_WIDGET
        return idaapi.AST_DISABLE_FOR_WIDGET


class FunctionRetrohuntDisassemblyViewHook(idaapi.UI_Hooks):
    def finish_populating_widget_popup(self, widget, popup):
        tft = idaapi.get_widget_type(widget)
        if tft == idaapi.BWN_DISASM:
            idaapi.attach_action_to_popup(
                widget,
                popup,
                FunctionRetrohuntDisassemblyViewHandler.action_name,
                f"{PLUGIN_NAME}/",
                idaapi.SETMENU_APP
            )
