import idaapi
import sip
from PyQt5 import QtWidgets

from threatray_ida_plugin.application.signature_management_service import SignatureManagementService
from threatray_ida_plugin.handlers.threatray_handler import ThreatrayHandler
from threatray_ida_plugin.libs.constants import PLUGIN_ID, PLUGIN_NAME, OKAY_RESPONSE


class SignatureManagementHandler(ThreatrayHandler):
    action_name = f'{PLUGIN_ID}:signature_management'

    def __init__(self, service: SignatureManagementService):
        super().__init__(
            action_name=self.action_name,
            action_text='Signature Management',
            wanted_hotkey='',
            help='Add or remove functions to/from a signature')
        self.__service = service

    def activate(self, ctx) -> int:
        twidget = int(ctx.widget)
        widget = sip.wrapinstance(int(twidget), QtWidgets.QWidget)
        table = widget.findChild(QtWidgets.QTableView)
        selected_funcs = [s for s in table.selectionModel().selectedRows()]

        selected_functions = []
        for func in selected_funcs:
            address = func.sibling(func.row(), 2).data()
            selected_functions.append(int(address, 16))

        self.__service.workflow(selected_functions)
        return OKAY_RESPONSE

    def update(self, ctx) -> int:
        if ctx.widget_type == idaapi.BWN_FUNCS:
            return idaapi.AST_ENABLE_FOR_WIDGET
        return idaapi.AST_DISABLE_FOR_WIDGET


class SignatureManagerHook(idaapi.UI_Hooks):
    def finish_populating_widget_popup(self, widget, popup):
        tft = idaapi.get_widget_type(widget)
        if tft == idaapi.BWN_FUNCS:
            idaapi.attach_action_to_popup(
                widget,
                popup,
                SignatureManagementHandler.action_name,
                f"{PLUGIN_NAME}/",
                idaapi.SETMENU_APP
            )
