from dataclasses import dataclass, replace
from datetime import datetime
from typing import List, Dict


@dataclass(frozen=True)
class FunctionRetrohuntCodeRegion:
    # code region object of the api response except the uid collection
    analysis_id: str
    pid: int
    base: int
    hash_sha256: str
    nr_of_function_matches: int
    threats: List[str]
    verdict: str
    analysis_created_at: str


@dataclass(frozen=True)
class FunctionRetrohuntCodeRegionWithUids(FunctionRetrohuntCodeRegion):
    matching_uids: List[str]  # might be UID or hex address

    def get_key(self) -> str:
        return f'{self.analysis_id}_{self.pid}_{self.base}_{self.hash_sha256}'

    def add_matching_uid(self, uid: str) -> 'FunctionRetrohuntCodeRegionWithUids':
        new_matching_uids = set(self.matching_uids)
        new_matching_uids.add(uid)
        return replace(self, matching_uids=list(new_matching_uids))

    def add_nr_of_function_matches(self, nr_of_function_matches) -> 'FunctionRetrohuntCodeRegionWithUids':
        return replace(self, nr_of_function_matches=self.nr_of_function_matches + nr_of_function_matches)

    @staticmethod
    def from_dict(data: Dict, created_at: str = '') -> 'FunctionRetrohuntCodeRegionWithUids':
        return FunctionRetrohuntCodeRegionWithUids(analysis_id=data['analysis_id'],
                                                   hash_sha256=data['hash_sha256'],
                                                   pid=data['pid'],
                                                   base=data['base'],
                                                   nr_of_function_matches=data['nr_of_function_matches'],
                                                   threats=[threat['label'] for threat in data['threats']],
                                                   verdict=data['verdict'],
                                                   matching_uids=[],
                                                   analysis_created_at=created_at
                                                   )


@dataclass(frozen=True)
class FunctionRetrohuntCodeRegionWithFunctions(FunctionRetrohuntCodeRegion):
    matching_input_functions: List[str]

    @staticmethod
    def from_function_retrohunt_code_region_with_uids(
            function_retrohunt_with_uids: FunctionRetrohuntCodeRegionWithUids,
            matching_input_functions: List[str]) -> 'FunctionRetrohuntCodeRegionWithFunctions':
        return FunctionRetrohuntCodeRegionWithFunctions(
            analysis_id=function_retrohunt_with_uids.analysis_id,
            pid=function_retrohunt_with_uids.pid,
            base=function_retrohunt_with_uids.base,
            hash_sha256=function_retrohunt_with_uids.hash_sha256,
            nr_of_function_matches=function_retrohunt_with_uids.nr_of_function_matches,
            threats=function_retrohunt_with_uids.threats,
            verdict=function_retrohunt_with_uids.verdict,
            matching_input_functions=matching_input_functions,
            analysis_created_at=function_retrohunt_with_uids.analysis_created_at
        )
