from typing import List
from datetime import datetime

import idaapi
from PyQt5 import QtWidgets

from threatray_ida_plugin.libs.constants import OKAY_RESPONSE
from threatray_ida_plugin.models.function_retrohunt_result import FunctionRetrohuntCodeRegionWithFunctions
from threatray_ida_plugin.views.utils import get_widget_name


class FunctionRetrohuntResultView(idaapi.PluginForm):
    def __init__(self, result: List[FunctionRetrohuntCodeRegionWithFunctions]):
        super().__init__()
        self.__result: List[FunctionRetrohuntCodeRegionWithFunctions] = result

    def OnCreate(self, form) -> int:
        main_widget = idaapi.find_widget('IDA View-A')
        idaapi.activate_widget(main_widget, True)
        self.parent = self.FormToPyQtWidget(form)
        self.__create()
        return OKAY_RESPONSE

    def __create(self):
        header: List[str] = ['Analysis Id', 'Analysis Created', 'PID', 'Base', 'Binary File Hash', 'Binary File Verdict',
                             'Binary File Threats', 'Nr. Of Function Matches', 'Matching Functions']

        layout = QtWidgets.QGridLayout(self.parent)
        table = QtWidgets.QTableWidget(0, len(header))
        layout.addWidget(table)
        table.setHorizontalHeaderLabels(header)
        table.horizontalHeader().setSectionResizeMode(QtWidgets.QHeaderView.ResizeToContents)

        for row_index, result in enumerate(sorted(self.__result, key=sort_by_date, reverse=True)):
            result: FunctionRetrohuntCodeRegionWithFunctions
            table.insertRow(row_index)
            for column_index, item in enumerate((result.analysis_id,
                                                 result.analysis_created_at,
                                                 str(result.pid),
                                                 hex(result.base),
                                                 result.hash_sha256,
                                                 result.verdict,
                                                 str(result.threats),
                                                 str(result.nr_of_function_matches),
                                                 str(result.matching_input_functions))):
                table.setItem(row_index, column_index, QtWidgets.QTableWidgetItem(item))

    def Show(self) -> idaapi.PluginForm.Show:
        return idaapi.PluginForm.Show(self,
                                      get_widget_name('Function Retrohunt Results'),
                                      options=(idaapi.PluginForm.WOPN_PERSIST
                                               | idaapi.PluginForm.WCLS_SAVE
                                               | idaapi.PluginForm.WOPN_MENU
                                               | idaapi.PluginForm.WOPN_RESTORE
                                               | idaapi.PluginForm.WOPN_TAB),
                                      )

    @staticmethod
    def create(result: List[FunctionRetrohuntCodeRegionWithFunctions]):
        f = FunctionRetrohuntResultView(result)
        f.Show()


def sort_by_date(func: FunctionRetrohuntCodeRegionWithFunctions):
    return datetime.fromisoformat(func.analysis_created_at)
