from enum import Enum
from typing import Optional

import idaapi

from threatray_ida_plugin.libs.constants import OKAY_RESPONSE

TEXT = r"""BUTTON YES* Select
BUTTON NO NONE
BUTTON CANCEL Cancel
Select action for update signature

{form_change}
<Action:{actions}>
"""


class SignatureAction(Enum):
    ADD = 'add'
    REMOVE = 'remove'


class SignatureActionBox(idaapi.Form):
    def __init__(self):
        self.canceled: bool = False
        self.actions_list = [SignatureAction.ADD.value, SignatureAction.REMOVE.value]
        self.selected_action = None

        idaapi.Form.__init__(
            self,
            TEXT,
            {
                'form_change': idaapi.Form.FormChangeCb(self.on_form_change),
                'actions': idaapi.Form.DropdownListControl(
                    items=self.actions_list,
                    readonly=True,
                    selval=0
                ),
            }
        )

    def on_form_change(self, fid):
        self.selected_action = self.actions_list[self.GetControlValue(self.actions)]
        if fid == idaapi.CB_CANCEL:
            self.canceled = True
        return OKAY_RESPONSE

    @staticmethod
    def create() -> Optional[str]:
        box = SignatureActionBox()
        box.Compile()
        box.Execute()
        if box.canceled:
            return None
        return box.selected_action
