from typing import Optional, List

import idaapi

from threatray_ida_plugin.libs.constants import OKAY_RESPONSE

TEXT = r"""BUTTON YES* Select
BUTTON NO NONE
BUTTON CANCEL Cancel
Insert signature label

{form_change}
<Signature label:{signature_label}>
"""


class SignatureLabelBox(idaapi.Form):
    def __init__(self, labels: List[str]):
        self.canceled: bool = False
        self.selected_label = ''
        self.__labels = labels

        idaapi.Form.__init__(
            self,
            TEXT,
            {
                'form_change': idaapi.Form.FormChangeCb(self.on_form_change),
                'signature_label': idaapi.Form.DropdownListControl(
                    items=self.__labels,
                    readonly=False,
                    selval=self.selected_label,
                    width=150,  # it does not impact the visual in IDA8.3
                    swidth=150,  # it does not impact the visual in IDA8.3
                ),
            }
        )

    def on_form_change(self, fid):
        self.selected_label = self.GetControlValue(self.signature_label)

        if fid == idaapi.CB_CANCEL:
            self.canceled = True

        return OKAY_RESPONSE

    @staticmethod
    def create(labels: List[str]) -> Optional[str]:
        box = SignatureLabelBox(labels)
        box.Compile()
        box.Execute()
        if box.canceled:
            return None
        return box.selected_label.strip()
